/*-
 * Copyright (C)2005..2014 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C)2005..2014 @BABOLO http://www.babolo.ru/\n\
@(#)All rights reserved.\n";
static const char rcsid[] = "$Id: test.c,v 1.8 2014/09/06 11:29:21 babolo Exp $";
#endif /* not lint */

#define BLIN_COMPAT  3
#define Bpars_COMPAT 3
#define MULAR_COMPAT 0

#include <sys/types.h>
#include <sysexits.h>
#include <unistd.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <err.h>
#include <babolo/BLINflag.h>
#include <babolo/parser.h>
#include "../multilar.h"

int
main(int argc, char **argv) {
    int level, i, ex = EX_OK;
    mular_descriptor *mular;
    u_int32_t j, cflag = 0;
    babolo_opts *bos;
    FILE *out = NULL;
    BLIN_flag flags;
    const char **c, *d;

    level = 0;
    flags = BLIN_BIT0 | MULAR_CHAR | MULAR_HEXU | MULAR_DUAD | MULAR_SWP1 | MULAR_SWP2;
#   define blin_internal_flags (flags & BLIN_MASK)
    if  (!argv) {
        ifBLIN_QX0("No argv");
        ex = EX_USAGE;
        goto out;
    }
    if  (!(bos = babolo_openopts(Bpars_NONU | Bpars_SYME, 'R'))) {
        ifBLIN_QW0("babolo_openopts");
        ex = EX_SOFTWARE;
        goto out;
    }
    if  (babolo_setopts(bos, Bpars_NONU, argc, argv, "fhl:qo:stuvz")) {
        ifBLIN_QW0("babolo_setopts");
        ex = EX_SOFTWARE;
        goto out;
    }
    while (0 < (i = babolo_getopts(bos))) {
        switch (i) {
        case 'f':
            cflag++;
            break;
        case 'h':
            flags |= MULAR_HALF;
            break;
        case 'l':
            errno = 0;
            level = babolo_getoptunum(bos, 0);
            if  (errno) {
                ifBLIN_QW0("babolo_getoptunum");
                ex = EX_SOFTWARE;
                goto out;
            }
            break;
        case 'q':
            BLIN_QUIET(flags);
            break;
        case 's':
            flags |= MULAR_STRI;
            break;
        case 'o':
            if  (!(out = fopen(babolo_getoptsarg(bos), "a"))) {
                ifBLIN_QW0("fopen(babolo_getoptsarg())");
                ex = EX_IOERR;
                goto out;
            }
            break;
        case 't':
            flags &= ~(MULAR_HEXU | MULAR_DUAD);
            break;
        case 'u':
            flags |= MULAR_UPPE;
            break;
        case 'v':
            BLIN_VERBOSE(flags);
            break;
        case 'z':
            flags |= MULAR_ZERO;
            break;
        default:
            exit(EX_USAGE);
    }   }
    if  (!out) out = stderr;
    if  (!(mular = mular_create(flags, level, 0, NULL))) {
        ifBLIN_QW0("mular_create");
        ex = EX_IOERR;
        goto out;
    }
    for (i = 0; i < level; i++) mular->size[i] = strtoul(babolo_getargs(bos), NULL, 0);
    if  (!cflag) mular->flags &= ~(BLIN_BIT2 | BLIN_BIT3 | BLIN_BIT4);
    for (i = 0; !!(d = babolo_getargs(bos)); ++i) {
        ifBLIN_QO1 mular_dump(mular, out);
        for (j = 0; j < mular->top.index; j++) {
            c = (const char**)mular_getix(mular, j);
            if  (!c || strcmp(d, *c) <= 0) break;
        }
        if  (!cflag) mular->flags = flags;
        ifBLIN_QO0 fprintf(out, "###i=%d @@@%u=%s~\n", i, j, d);
        if  ((c = ((const char**)mular_insert(mular, j)))) {
            *c = d;
        } else {
            fprintf(out, " \n");
        }
        if  (!cflag) mular->flags &= ~(BLIN_BIT2 | BLIN_BIT3 | BLIN_BIT4);
    }
    mular_dump(mular, out);
    for (i = 0; (c = ((const char**)mular_getix(mular, i))); i++) {
        fprintf(out, "%d=", i);
        fprintf(out, "%s~\n", *c);
    }
    mular_destroy(mular);
out:
    exit(ex);
#   undef blin_internal_flags
}
